@@ -70,8 +70,9 @@ GST_DEBUG_CATEGORY (cheese_camera_device_cat);
 #define GST_CAT_DEFAULT cheese_camera_device_cat
 
 static const gchar * const supported_formats[] = {
-  "video/x-raw",
-  NULL
+    "video/x-raw",
+    "image/jpeg",
+    NULL
 };
 
 /* FIXME: make this configurable */
@@ -86,6 +87,7 @@ enum
 {
   PROP_0,
   PROP_NAME,
+  PROP_PATH,
   PROP_DEVICE,
   PROP_LAST
 };
@@ -98,6 +100,7 @@ typedef struct
   gchar     *name;
   GstCaps   *caps;
   GList     *formats; /* list members are CheeseVideoFormatFull structs. */
+  gchar     *path;
 
   GError    *construct_error;
 } CheeseCameraDevicePrivate;
@@ -162,6 +165,27 @@ compare_formats (gconstpointer a, gconstpointer b)
   return (d->width * d->height - c->width * c->height);
 }
 
+static GstCaps *
+format_caps (const gchar * const formats[])
+{
+    GstCaps *filter;
+    gsize i;
+
+    filter = gst_caps_new_empty ();
+
+    for (i = 0; formats[i] != NULL; i++)
+    {
+        gst_caps_append (filter,
+                         gst_caps_new_simple (formats[i],
+                                              "framerate",
+                                              GST_TYPE_FRACTION_RANGE,
+                                              0, 1, CHEESE_MAXIMUM_RATE, 1,
+                                              NULL));
+    }
+
+    return filter;
+}
+
 /*
  * cheese_camera_device_filter_caps:
  * @device: the #CheeseCameraDevice
@@ -181,18 +205,8 @@ cheese_camera_device_filter_caps (CheeseCameraDevice *device,
 {
   GstCaps *filter;
   GstCaps *allowed;
-  gsize i;
 
-  filter = gst_caps_new_empty ();
-
-  for (i = 0; formats[i] != NULL; i++)
-  {
-    gst_caps_append (filter,
-                     gst_caps_new_simple (formats[i],
-                                          "framerate", GST_TYPE_FRACTION_RANGE,
-                                          0, 1, CHEESE_MAXIMUM_RATE, 1,
-                                          NULL));
-  }
+    filter = format_caps (formats);
 
   allowed = gst_caps_intersect (caps, filter);
 
@@ -590,6 +604,9 @@ cheese_camera_device_get_property (GObject *object, guint prop_id, GValue *value
     case PROP_DEVICE:
       g_value_set_object (value, priv->device);
       break;
+    case PROP_PATH:
+      g_value_set_string (value, priv->path);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -614,6 +631,12 @@ cheese_camera_device_set_property (GObject *object, guint prop_id, const GValue
       priv->device = g_value_dup_object (value);
       g_free (priv->name);
       priv->name = gst_device_get_display_name (priv->device);
+      g_free (priv->path);
+      priv->path = g_value_dup_string (gst_structure_get_value (gst_device_get_properties (priv->device), "api.v4l2.path"));
+      break;
+    case PROP_PATH:
+      g_free (priv->path);
+      priv->path = g_value_dup_string (value);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -629,6 +652,7 @@ cheese_camera_device_finalize (GObject *object)
 
   g_object_unref (priv->device);
   g_free (priv->name);
+  g_free (priv->path);
 
   gst_caps_unref (priv->caps);
   free_format_list (device);
@@ -666,6 +690,18 @@ cheese_camera_device_class_init (CheeseCameraDeviceClass *klass)
                                                G_PARAM_CONSTRUCT_ONLY |
                                                G_PARAM_STATIC_STRINGS);
 
+  /**
+   * CheeseCameraDevice:path:
+   *
+   * Path of the video capture device.
+   */
+  properties[PROP_PATH] = g_param_spec_string ("path",
+                                               "Device path",
+                                               "Path of the video capture device",
+                                               NULL,
+                                               G_PARAM_READWRITE |
+                                               G_PARAM_CONSTRUCT_ONLY |
+                                               G_PARAM_STATIC_STRINGS);
   /**
    * CheeseCameraDevice:device:
    *
@@ -769,6 +805,25 @@ cheese_camera_device_get_format_list (CheeseCameraDevice *device)
     return g_list_copy (priv->formats);
 }
 
+/**
+ * cheese_camera_device_get_path:
+ * @device: a #CheeseCameraDevice
+ *
+ * Get path for the device, as reported by udev.
+ *
+ * Returns: (transfer none): the path of the video capture device
+ */
+const gchar *
+cheese_camera_device_get_path (CheeseCameraDevice *device)
+{
+  CheeseCameraDevicePrivate *priv;
+
+  g_return_val_if_fail (CHEESE_IS_CAMERA_DEVICE (device), NULL);
+
+  priv = cheese_camera_device_get_instance_private (device);
+
+  return priv->path;
+}
 /**
  * cheese_camera_device_get_name:
  * @device: a #CheeseCameraDevice
@@ -892,10 +947,9 @@ cheese_camera_device_get_caps_for_format (CheeseCameraDevice *device,
                                           CheeseVideoFormat  *format)
 {
     CheeseCameraDevicePrivate *priv;
-  CheeseVideoFormatFull *full_format;
-  GstCaps *desired_caps;
-  GstCaps *subset_caps;
-  gsize i;
+    CheeseVideoFormatFull *full_format;
+    GstCaps *result_caps;
+    gsize i;
 
   g_return_val_if_fail (CHEESE_IS_CAMERA_DEVICE (device), NULL);
 
@@ -912,21 +966,39 @@ cheese_camera_device_get_caps_for_format (CheeseCameraDevice *device,
             full_format->width, full_format->height,
             full_format->fr_numerator, full_format->fr_denominator);
 
-  desired_caps = gst_caps_new_empty ();
-
-  for (i = 0; supported_formats[i] != NULL; i++)
-  {
-    gst_caps_append (desired_caps,
-                     cheese_camera_device_format_to_caps (supported_formats[i],
-                                                          full_format));
-  }
-
     priv = cheese_camera_device_get_instance_private (device);
-    subset_caps = gst_caps_intersect (desired_caps, priv->caps);
-  subset_caps = gst_caps_simplify (subset_caps);
-  gst_caps_unref (desired_caps);
 
-  GST_INFO ("Got %" GST_PTR_FORMAT, subset_caps);
+    result_caps = gst_caps_new_empty ();
 
-  return subset_caps;
+    for (i = 0; supported_formats[i] != NULL; i++)
+    {
+        GstCaps *desired_caps;
+        GstCaps *subset_caps;
+
+        desired_caps = cheese_camera_device_format_to_caps (supported_formats[i],
+                                                            full_format);
+        subset_caps = gst_caps_intersect (desired_caps, priv->caps);
+        subset_caps = gst_caps_simplify (subset_caps);
+
+        gst_caps_append (result_caps, subset_caps);
+
+        gst_caps_unref (desired_caps);
+    }
+
+    GST_INFO ("Got %" GST_PTR_FORMAT, result_caps);
+
+    return result_caps;
+}
+
+/**
+ * cheese_camera_device_supported_format_caps:
+ *
+ * Get the #GstCaps that are supported for all #CheeseCameraDevice
+ *
+ * Returns: (transfer full): the #GstCaps
+ */
+GstCaps *
+cheese_camera_device_supported_format_caps (void)
+{
+    return format_caps (supported_formats);
 }
