@@ -92,7 +92,7 @@ GtkWidget * cheese_thumb_view_new (void);
 static gboolean
 cheese_thumb_view_idle_append_item (gpointer data)
 {
-  CheeseThumbViewIdleData *item = g_queue_peek_head (data);
+  CheeseThumbViewIdleData *item = g_queue_pop_head (data);
   CheeseThumbView         *thumb_view;
   CheeseThumbViewPrivate  *priv;
 
@@ -113,12 +113,15 @@ cheese_thumb_view_idle_append_item (gpointer data)
   char                         *mime_type;
   char                         *uri;
   char                         *filename;
+  GError                       *error = NULL;
 
-  info = g_file_query_info (file, "standard::content-type,time::modified", 0, NULL, NULL);
+  info = g_file_query_info (file, "standard::content-type,time::modified", 0, NULL, &error);
 
   if (!info)
   {
-    g_warning ("Invalid filename\n");
+    g_warning ("Could not query file info: %s", error->message);
+    g_error_free (error);
+    g_slice_free (CheeseThumbViewIdleData, item);
     return TRUE;
   }
   g_file_info_get_modification_time (info, &mtime);
@@ -167,6 +170,7 @@ cheese_thumb_view_idle_append_item (gpointer data)
     if (error)
     {
       g_warning ("%s", error->message);
+      g_slice_free (CheeseThumbViewIdleData, item);
       return TRUE;
     }
   }
@@ -183,7 +187,6 @@ cheese_thumb_view_idle_append_item (gpointer data)
   g_object_unref (pixbuf);
   g_object_unref (file);
   g_slice_free (CheeseThumbViewIdleData, item);
-  g_queue_pop_head (data);
 
   return TRUE;
 }
